# Copyright (C) 2024,2025,2026 Kian-Meng Ang
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.


def test_crop_subcommand_success(cli_runner, image_file):
    img_path = image_file("sample.png")

    ret = cli_runner(
        "--overwrite",
        "crop",
        str(img_path),
        "--box",
        "50,50,150,150",
        "--output-dir",
        ".",
    )
    assert ret.returncode == 0


def test_crop_subcommand_missing_box_failure(cli_runner, image_file):
    img_path = image_file("sample.png")
    ret = cli_runner("crop", str(img_path))
    assert ret.returncode != 0
    assert (
        "error: the following arguments are required: -b/--box" in ret.stderr
    )


def test_crop_subcommand_invalid_box_format_failure(cli_runner, image_file):
    img_path = image_file("sample.png")
    ret = cli_runner("crop", str(img_path), "--box", "10,10,100")
    assert ret.returncode != 0
    assert (
        "error: invalid box format: 10,10,100. "
        "Box must contain exactly four comma-separated integers." in ret.stderr
    )

    ret = cli_runner("crop", str(img_path), "--box", "10,10,100,abc")
    assert ret.returncode != 0
    assert (
        "error: invalid box format: 10,10,100,abc. invalid literal for int()"
        in ret.stderr
    )


def test_crop_subcommand_invalid_coordinates_failure(cli_runner, image_file):
    img_path = image_file("sample.png")
    # left more or equal right
    ret = cli_runner("crop", str(img_path), "--box", "100,100,50,200")
    assert ret.returncode != 0
    assert (
        "error: invalid box format: 100,100,50,200. "
        "Left coordinate must be less than right, and "
        "upper must be less than lower." in ret.stderr
    )

    # upper more or equal lower
    ret = cli_runner("crop", str(img_path), "--box", "50,200,100,100")
    assert ret.returncode != 0
    assert (
        "error: invalid box format: 50,200,100,100. "
        "Left coordinate must be less than right, and "
        "upper must be less than lower." in ret.stderr
    )



# Copyright (C) 2024,2025,2026 Kian-Meng Ang

# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.


def test_resize_subcommand(cli_runner, image_file):
    img_path = image_file("sample.png")
    ret = cli_runner("resize", str(img_path), "--width", "200")
    assert ret.returncode == 0


def test_resize_subcommand_aspect_ratio_success(cli_runner, image_file):
    img_path = image_file("sample.png")
    ret = cli_runner(
        "resize",
        str(img_path),
        "--width",
        "320",
        "--aspect-ratio",
        "16:9",
    )
    assert ret.returncode == 0


def test_resize_subcommand_aspect_ratio_missing_dimension_failure(
    cli_runner,
    image_file,
):
    img_path = image_file("sample.png")
    ret = cli_runner("resize", str(img_path), "--aspect-ratio", "16:9")
    assert ret.returncode != 0
    assert (
        "error: argument -ar/--aspect-ratio requires "
        "either -W/--width or -H/--height" in ret.stderr
    )


def test_resize_subcommand_mutually_exclusive_failure(cli_runner, image_file):
    img_path = image_file("sample.png")
    ret = cli_runner(
        "resize",
        str(img_path),
        "--width",
        "200",
        "--height",
        "200",
    )
    assert ret.returncode != 0
    assert (
        "error: argument -W/--width and -H/--height are mutually "
        "exclusive when not using --canvas" in ret.stderr
    )


def test_resize_subcommand_canvas_required_failure(cli_runner, image_file):
    img_path = image_file("sample.png")
    ret = cli_runner("resize", str(img_path), "--canvas", "--width", "200")
    assert ret.returncode != 0
    assert (
        "error: argument -W/--width and -H/--height are "
        "required when using --canvas" in ret.stderr
    )

    ret = cli_runner("resize", str(img_path), "--canvas", "--height", "200")
    assert ret.returncode != 0
    assert (
        "error: argument -W/--width and -H/--height are "
        "required when using --canvas" in ret.stderr
    )


def test_resize_subcommand_canvas_success(cli_runner, image_file):
    img_path = image_file("sample.png")
    ret = cli_runner(
        "resize",
        str(img_path),
        "--canvas",
        "--width",
        "200",
        "--height",
        "200",
    )
    assert ret.returncode == 0



# Copyright (C) 2024,2025,2026 Kian-Meng Ang
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

"""Auto subcommand."""

from __future__ import annotations

import argparse
import logging
from pathlib import Path
from typing import Any

import fotolab.subcommands.animate
import fotolab.subcommands.contrast
import fotolab.subcommands.resize
import fotolab.subcommands.sharpen
import fotolab.subcommands.watermark
from fotolab import open_image

from .common import add_common_arguments, log_args_decorator

log = logging.getLogger(__name__)


def build_subparser(subparsers: argparse._SubParsersAction[Any]) -> None:
    """Build the subparser."""
    auto_parser = subparsers.add_parser(
        "auto",
        help="auto adjust (resize, contrast, and watermark) a photo",
    )

    auto_parser.set_defaults(func=run)

    add_common_arguments(auto_parser)

    auto_parser.add_argument(
        "-t",
        "--title",
        dest="title",
        help="set the tile (default: '%(default)s')",
        type=str,
        default=None,
        metavar="TITLE",
    )

    auto_parser.add_argument(
        "-w",
        "--watermark",
        dest="watermark",
        help="set the watermark (default: '%(default)s')",
        type=str,
        default="kianmeng.org",
        metavar="WATERMARK_TEXT",
    )


def _get_base_filename(
    title: str | None,
    image_file: Path,
    *,
    multi: bool = False,
) -> str:
    """Generate a sanitized base filename from title or image filename."""
    if title:
        sanitized = title.lower().replace(",", "").replace(" ", "_")
        if multi:
            return f"{sanitized}_{image_file.stem}"
        return sanitized
    return image_file.stem


@log_args_decorator
def run(args: argparse.Namespace) -> None:
    """Run auto subcommand.

    Args:
        args (argparse.Namespace): Config from command line arguments

    Returns:
        None
    """
    text = args.watermark
    if args.title and args.watermark:
        text = f"{args.title}\n{args.watermark}"

    extra_args = {
        "width": 600,
        "height": 277,
        "cutoff": 1,
        "radius": 1,
        "percent": 100,
        "threshold": 2,
        "text": text,
        "position": "bottom-left",
        "font_size": 12,
        "font_color": "white",
        "outline_width": 2,
        "outline_color": "black",
        "padding": 15,
        "padding_x": None,
        "padding_y": None,
        "camera": False,
        "canvas": False,
        "lowercase": True,
        "before_after": False,
        "alpha": 128,
        "aspect_ratio": None,
    }

    # Base configuration for all steps
    base_args_dict = {**vars(args), **extra_args}
    base_args_dict["open"] = False  # Disable opening intermediates

    processed_paths = []
    output_dir = Path(args.output_dir)
    output_dir.mkdir(parents=True, exist_ok=True)

    is_multi = len(args.image_paths) > 1

    for image_path_str in args.image_paths:
        image_path = Path(image_path_str)
        base_name = _get_base_filename(args.title, image_path, multi=is_multi)
        final_filename = f"{base_name}{image_path.suffix.lower()}"
        final_output_path = output_dir / final_filename

        # Step 1: Resize (Read original, save to output directory)
        step_args = argparse.Namespace(**base_args_dict)
        step_args.image_paths = [str(image_path)]
        step_args.output_filename = final_filename
        step_args.overwrite = False
        fotolab.subcommands.resize.run(step_args)

        # Subsequent Steps: Contrast, Sharpen, Watermark
        # (Cumulatively overwrite)
        step_args.image_paths = [str(final_output_path)]
        step_args.overwrite = True
        step_args.output_filename = None

        fotolab.subcommands.contrast.run(step_args)
        fotolab.subcommands.sharpen.run(step_args)
        fotolab.subcommands.watermark.run(step_args)

        processed_paths.append(final_output_path)

    # Final actions
    if is_multi:
        # Create animation from processed images
        gif_base = _get_base_filename(
            args.title,
            Path("animation"),
            multi=False,
        )
        gif_filename = f"{gif_base}.gif"

        animate_args = argparse.Namespace(**base_args_dict)
        animate_args.image_paths = [str(p) for p in processed_paths]
        animate_args.format = "gif"
        animate_args.duration = 2500
        animate_args.loop = 0
        animate_args.output_filename = gif_filename
        animate_args.overwrite = False
        fotolab.subcommands.animate.run(animate_args)

        if args.open:
            open_image(output_dir / gif_filename)
    elif args.open and processed_paths:
        open_image(processed_paths[0])



# Copyright (C) 2024,2025,2026 Kian-Meng Ang

# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

import platform
import sys

from fotolab import __version__


def test_env_output(cli_runner):
    ret = cli_runner("env")

    actual_sys_version = sys.version.replace("\n", "")
    actual_platform = platform.platform()

    expected_output = (
        f"fotolab: {__version__}\n"
        f"python: {actual_sys_version}\n"
        f"platform: {actual_platform}\n"
    )

    assert ret.stdout == expected_output
    assert ret.stderr == ""
    assert ret.returncode == 0



# Copyright (C) 2024,2025,2026 Kian-Meng Ang

# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.


from PIL import Image


def test_montage_subcommand(cli_runner, image_file):
    img_path = image_file("sample.png")
    ret = cli_runner("montage", str(img_path), str(img_path))
    assert ret.returncode == 0


def test_montage_subcommand_with_single_image_raises_error(
    cli_runner,
    image_file,
):
    img_path = image_file("sample.png")
    ret = cli_runner("montage", str(img_path))
    assert ret.returncode != 0
    assert "error: at least two images is required for montage" in ret.stdout


def test_montage_subcommand_with_border(cli_runner, image_file, tmp_path):
    img_path = image_file("sample.png")

    with Image.open(img_path) as img:
        orig_width, orig_height = img.size

    border_width = 10
    ret = cli_runner(
        "montage",
        "-b",
        str(border_width),
        str(img_path),
        str(img_path),
    )
    assert ret.returncode == 0

    full_output_path = (
        tmp_path / "scripttest" / "output" / "montage_sample.png"
    )

    assert full_output_path.exists()

    with Image.open(full_output_path) as img:
        width, height = img.size

        expected_width = 2 * (orig_width + 2 * border_width)
        expected_height = orig_height + 2 * border_width

        assert width == expected_width
        assert height == expected_height


def test_montage_subcommand_with_border_and_color(
    cli_runner,
    image_file,
    tmp_path,
):
    img_path = image_file("sample.png")
    border_width = 5
    border_color = "red"
    ret = cli_runner(
        "montage",
        "-b",
        str(border_width),
        "-bc",
        border_color,
        str(img_path),
        str(img_path),
    )
    assert ret.returncode == 0

    full_output_path = (
        tmp_path / "scripttest" / "output" / "montage_sample.png"
    )
    assert full_output_path.exists()

    with Image.open(full_output_path) as img:
        # check a pixel that should be in the border (top-left corner)
        # if image is RGBA, it might return (255, 0, 0, 255)
        pixel = img.getpixel((0, 0))
        assert pixel[:3] == (255, 0, 0)



# Copyright (C) 2024,2025,2026 Kian-Meng Ang

# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.


import argparse
from pathlib import Path

import fotolab.subcommands.auto


def test_auto_subcommand_single_image(cli_runner, image_file):
    img_path = image_file("sample.png")
    ret = cli_runner("auto", str(img_path))
    assert ret.returncode == 0
    assert "output/sample.png" in ret.files_created


def test_auto_subcommand_with_title(cli_runner, image_file):
    img_path = image_file("sample.png")
    ret = cli_runner("auto", str(img_path), "--title", "My Testing")
    assert ret.returncode == 0
    assert "output/my_testing.png" in ret.files_created


def test_auto_subcommand_multi_image(cli_runner, image_file):
    img_path1 = image_file("sample.png")
    img_path2 = Path(img_path1.parent, "sample2.png")
    img_path2.write_bytes(img_path1.read_bytes())

    ret = cli_runner(
        "auto",
        str(img_path1),
        str(img_path2),
        "--title",
        "Batch Job",
    )
    assert ret.returncode == 0

    assert "output/batch_job_sample.png" in ret.files_created
    assert "output/batch_job_sample2.png" in ret.files_created
    assert "output/batch_job.gif" in ret.files_created


def test_auto_subcommand_no_title_multi(cli_runner, image_file):
    img_path1 = image_file("sample.png")
    img_path2 = Path(img_path1.parent, "sample2.png")
    img_path2.write_bytes(img_path1.read_bytes())

    ret = cli_runner("auto", str(img_path1), str(img_path2))
    assert ret.returncode == 0

    assert "output/sample.png" in ret.files_created
    assert "output/sample2.png" in ret.files_created
    assert "output/animation.gif" in ret.files_created


def test_auto_subcommand_open_single(image_file, monkeypatch):
    mock_open = []
    monkeypatch.setattr(
        fotolab.subcommands.auto,
        "open_image",
        mock_open.append,
    )

    img_path = image_file("sample.png")
    args = argparse.Namespace(
        image_paths=[str(img_path)],
        open=True,
        title=None,
        watermark="kianmeng.org",
        output_dir="output",
        overwrite=False,
    )

    fotolab.subcommands.auto.run(args)
    assert len(mock_open) == 1


def test_auto_subcommand_open_multi(image_file, monkeypatch):
    mock_open = []
    monkeypatch.setattr(
        fotolab.subcommands.auto,
        "open_image",
        mock_open.append,
    )

    img_path1 = image_file("sample.png")
    img_path2 = Path(img_path1.parent, "sample2.png")
    img_path2.write_bytes(img_path1.read_bytes())

    args = argparse.Namespace(
        image_paths=[str(img_path1), str(img_path2)],
        open=True,
        title=None,
        watermark="kianmeng.org",
        output_dir="output",
        overwrite=False,
    )

    fotolab.subcommands.auto.run(args)
    assert len(mock_open) == 1
    assert "animation.gif" in str(mock_open[0])
